/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <config.h>
#include <nessuslib.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include <gtk/gtk.h>

#include "attack.h"
#include "xstuff.h"
#include "comm.h"
#include "auth.h"
#include "parser.h"
#include "monitor_dialog.h"
#include "plugin_list.h"
#include "xpm/connected.xpm"
struct attack_entries {
  GtkWidget * window;
  GtkWidget * hostname;
  GtkWidget * max_hosts;
};
extern int nessusd_running;

void attack_dialog_callback(GtkWidget * , struct attack_entries *);
void attack_input_callback(void * , int, int);


int attack_close(GtkWidget *, GtkWidget *);
void attack_cancel(GtkWidget *, GtkWidget *);

int
attack_close(a,b)
 GtkWidget *a,*b;
{
 nessusd_running = 0;
 return(FALSE);
}

void
attack_cancel(a,b)
 GtkWidget * a,*b;
{
 nessusd_running = 0;
 gtk_widget_hide(b);
}

/*
 * attack_dialog_setup
 *
 * This function draws the dialog which asks
 * the user the name of the host to probe
 *
 * Params :
 *   hostname : name of the nessusd host we are connected to
 * 
 * Returns : 0
 */ 
int 
attack_dialog_setup(hostname)
     char * hostname;
{
  struct attack_entries entries;
  GtkWidget * table;
  GtkWidget * button;
  GtkStyle  * style;
  GtkWidget * label;
  GtkWidget * pixmapwid;
  GdkPixmap * pixmap;
  GdkBitmap * mask;
  GtkWidget * box,*vbox;
  GtkWidget * separator;
  char * msg;
  
  entries.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect(GTK_OBJECT(entries.window), "destroy", 
		     GTK_SIGNAL_FUNC(attack_close),entries.window);
  gtk_signal_connect(GTK_OBJECT(entries.window), "delete_event", 
		     GTK_SIGNAL_FUNC(attack_close),NULL);		 
  gtk_window_set_title(GTK_WINDOW(entries.window), "Select host to test");
  gtk_container_border_width(GTK_CONTAINER(entries.window), 10);
  
  table = gtk_table_new(4, 3, TRUE);
  
  gtk_container_add(GTK_CONTAINER(entries.window), table);
  gtk_table_set_col_spacing(GTK_TABLE(table), 0, 15);
  gtk_table_set_col_spacing(GTK_TABLE(table), 1, 15);
  
  gtk_table_set_row_spacings(GTK_TABLE(table),15);
  label = gtk_label_new("Host to test :");
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0,1,1,2);
  gtk_widget_show(label);
  
  entries.hostname = gtk_entry_new();
  gtk_table_attach_defaults(GTK_TABLE(table), entries.hostname, 1,3,1,2);
  gtk_widget_show(entries.hostname);
  
  label = gtk_label_new("Max number of hosts to test :");
  gtk_table_attach_defaults(GTK_TABLE(table), label, 0,1,2,3);
  gtk_widget_show(label);
  
  entries.max_hosts = gtk_entry_new();
  gtk_entry_set_text(GTK_ENTRY(entries.max_hosts),"infinity");
  gtk_table_attach_defaults(GTK_TABLE(table), entries.max_hosts, 1,3,2,3);
  gtk_widget_show(entries.max_hosts);
  
  button = gtk_button_new_with_label("Plugin list");
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_signal_connect_object(GTK_OBJECT (button), "clicked", 
			    (GtkSignalFunc)plugin_list_setup,NULL);
  gtk_table_attach_defaults(GTK_TABLE(table), button, 0,1,3,4);
  gtk_widget_show(button);
  
  button = gtk_button_new_with_label("Cancel");
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_table_attach_defaults(GTK_TABLE(table), button, 1,2,3,4);
  gtk_signal_connect_object(GTK_OBJECT (button), "clicked", 
			    GTK_SIGNAL_FUNC(attack_cancel),
			    (void *)entries.window);
  gtk_widget_show(button);
  
  button = gtk_button_new_with_label("Go !");
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_table_attach_defaults(GTK_TABLE(table), button, 2,3,3,4);
  gtk_signal_connect_object(GTK_OBJECT (button), "clicked", 
			    (GtkSignalFunc)attack_dialog_callback,
			    (void*)&entries);
  gtk_widget_grab_default (button);
  gtk_widget_show(button);			   
  gtk_widget_realize(entries.window);
			    
  style = gtk_widget_get_style(entries.window);
  pixmap = gdk_pixmap_create_from_xpm_d(entries.window->window, &mask,
					&style->bg[GTK_STATE_NORMAL],
					connected_xpm); 
  pixmapwid = gtk_pixmap_new(pixmap, mask);	
  gtk_widget_show(pixmapwid);
  
  msg = emalloc(40+strlen(hostname));
  sprintf(msg, "Connected to %s", hostname);
  
  label = gtk_label_new(msg);
  vbox = gtk_vbox_new(FALSE,4);
  gtk_table_attach_defaults(GTK_TABLE(table),vbox, 0,3,0,1);
  box = gtk_hbox_new(FALSE,5);
  gtk_box_pack_start(GTK_BOX(vbox), box, FALSE,FALSE,0);
  gtk_widget_show(vbox);
  gtk_widget_show(box);
  separator = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(vbox), separator,FALSE,FALSE,0);
  gtk_widget_show(separator);
  gtk_box_pack_start(GTK_BOX(box), pixmapwid, FALSE,FALSE, 3);
  gtk_box_pack_start(GTK_BOX(box), label, FALSE,FALSE,3);
  gtk_widget_show(label);			   
  gtk_widget_show(button);
  gtk_widget_show(table);
  gtk_widget_show(entries.window);
  gtk_main();
  return(0);
}

/* 
 * attack_dialog_callback
 *
 * This function is called when the user clicks the
 * 'Go!' button of the attack dialog
 *
 */
void 
attack_dialog_callback(widget,entries)
     GtkWidget * widget;
     struct attack_entries * entries;
{
  char * hostname;
  char * t;
  char * max;
  int max_hosts;
  
  /* Read the user input ... */ 
  t = gtk_entry_get_text(GTK_ENTRY(entries->hostname));
  max = gtk_entry_get_text(GTK_ENTRY(entries->max_hosts));
  if(!strcmp(max, "infinity"))max_hosts = -1;
  else max_hosts = atoi(max);
  
  hostname = emalloc(strlen(t)+1);
  strncpy(hostname, t, strlen(t));
  gtk_widget_hide(entries->window);
  
  /* ... and setup the monitor window */
  monitor_dialog_setup(hostname, max_hosts);
}


