/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
 
 
#include <config.h>
#include <nessuslib.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <pwd.h>
#include <unistd.h>

#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include <gtk/gtk.h>

#include "attack_dialog.h"
#include "xstuff.h"
#include "nessus.h"
#include "error_dialog.h"
#include "xpm/computer.xpm"
#include "xpm/user.xpm"

extern nessusd_running;
struct auth_entries {
  GtkWidget * window;
  GtkWidget * username;
  GtkWidget * password;
  GtkWidget * hostname;
  GtkWidget * port;
};

	
void auth_dialog_callback(GtkWidget *, struct auth_entries *);
void auth_dialog_cancel(GtkWidget *, GtkWidget *);
int auth_dialog_close(GtkWidget *, GtkWidget *);	

static char *
get_username()
{
 char * user;
 struct passwd * pwd;
 /* Look up the user's name. */
  user = getenv ("USER");
  if (user)
    return user;

  user = getenv ("LOGNAME");
  if (user)
    return user;

  pwd = getpwuid (getuid ());
  if (pwd && pwd->pw_name)
    return pwd->pw_name;

  return "";
}

/*
 * auth_dialog_setup
 * 
 * This function draws the dialog window which prompts
 * the user for :
 *  - the host to connect to
 *  - the port to connect to
 *  - the login of the user
 *  - the password of the user
 * 
 * Params  : none
 * Returns : 0
 *
 */
int 
auth_dialog_setup()
{
  struct auth_entries entries;
  GtkWidget * label;
  GtkWidget * button;
  GtkWidget * table;
  GtkStyle *style;
  GtkWidget * pixmapwid;
  GdkPixmap * pixmap;
  GdkBitmap * mask;
  GtkWidget * box;        
  GtkWidget * separator;
         
  if(nessusd_running){
   	show_error("Can't start a new session :\nanother session is running !");
   	return(0);
   	}
  else nessusd_running = 1;
  entries.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW (entries.window), "New Session");
  gtk_signal_connect(GTK_OBJECT (entries.window), "destroy",
		     GTK_SIGNAL_FUNC(auth_dialog_close),entries.window);
  gtk_signal_connect(GTK_OBJECT(entries.window),"delete_event",
		     GTK_SIGNAL_FUNC(auth_dialog_close),entries.window);
  
  gtk_container_border_width(GTK_CONTAINER(entries.window), 10);
  
  table = gtk_table_new(8, 2, TRUE);
  gtk_container_add(GTK_CONTAINER (entries.window), table);
  
  gtk_table_set_col_spacing(GTK_TABLE(table), 0, 15);
  gtk_table_set_row_spacing(GTK_TABLE(table), 0, 15);
  
  label = gtk_label_new("New session setup :");
  separator = gtk_hseparator_new();
  box = gtk_vbox_new(FALSE,0);
  gtk_table_attach_defaults(GTK_TABLE(table), box, 0,2,0,1);
  gtk_widget_show(box);
  gtk_box_pack_start(GTK_BOX(box),label,FALSE,FALSE,5);
  gtk_box_pack_start(GTK_BOX(box),separator,FALSE,FALSE,2);
  gtk_widget_show(label);
  gtk_widget_show(separator); 
  separator = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(box),separator,FALSE,FALSE,0);
  gtk_widget_show(separator); 
  
#ifdef USE_AF_INET
  entries.hostname = gtk_entry_new_with_max_length (255);
#else
  entries.hostname = gtk_label_new("(this is AF_UNIX)");
#endif
  gtk_table_attach_defaults(GTK_TABLE(table), entries.hostname, 1,2,1,2);
  gtk_widget_show(entries.hostname);
  
  box = gtk_hbox_new(FALSE,0);
  gtk_table_attach_defaults(GTK_TABLE(table), box, 0,1,2,3);
  gtk_widget_show(box);
  label = gtk_label_new("Port : ");
  gtk_box_pack_end(GTK_BOX(box), label, FALSE, FALSE,0);
  gtk_widget_show(label);
  
#ifdef USE_AF_INET
  entries.port = gtk_entry_new_with_max_length (255);
  {
   char tbuf[10];
   sprintf (tbuf, "%d", DEFAULT_PORT);
   gtk_entry_set_text (GTK_ENTRY(entries.port), tbuf);
  }
#else 
  entries.port = gtk_label_new("(this is AF_UNIX)");
#endif
  gtk_table_attach_defaults(GTK_TABLE(table), entries.port, 1,2,2,3);
  gtk_widget_show(entries.port);
  
  separator = gtk_hseparator_new();
  gtk_table_attach_defaults(GTK_TABLE(table), separator, 0,2,3,4);
  gtk_widget_show(separator);
  
  entries.username = gtk_entry_new_with_max_length (255);
  gtk_entry_set_text (GTK_ENTRY(entries.username), get_username ());
  gtk_table_attach_defaults(GTK_TABLE(table), entries.username, 1,2,4,5);
  
  box = gtk_hbox_new(FALSE,0);
  gtk_table_attach_defaults(GTK_TABLE(table), box, 0,1,5,6);
  gtk_widget_show(box);
  
  label = gtk_label_new("Password :");
  gtk_box_pack_end(GTK_BOX(box), label, FALSE, FALSE,0);
  gtk_widget_show(label);
  
  entries.password = gtk_entry_new_with_max_length (255);
  gtk_entry_set_visibility(GTK_ENTRY(entries.password), FALSE);
  gtk_table_attach_defaults(GTK_TABLE(table), entries.password, 1,2,5,6);
  
  button = gtk_button_new_with_label("Cancel");
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_signal_connect_object(GTK_OBJECT (button), "clicked",
 			   (GtkSignalFunc)auth_dialog_cancel,(void*)entries.window);
  gtk_table_attach_defaults(GTK_TABLE(table), button, 0,1,7,8);
  gtk_widget_show(button);
  
  button = gtk_button_new_with_label("Connect");
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_table_attach_defaults(GTK_TABLE(table), button, 1,2,7,8);
  gtk_signal_connect_object(GTK_OBJECT (button), "clicked",
 			   (GtkSignalFunc)auth_dialog_callback,
 			   (void *)&entries);
  gtk_widget_grab_default (button);
  gtk_widget_show(button);
  box = gtk_hbox_new(FALSE,5);
  gtk_table_attach_defaults(GTK_TABLE(table), box, 0,1,1,2);
  gtk_widget_show(box);
 
  gtk_widget_realize(entries.window);
  style = gtk_widget_get_style(entries.window);
  pixmap = gdk_pixmap_create_from_xpm_d(entries.window->window, &mask,
					&style->bg[GTK_STATE_NORMAL],
					computer_xpm);             
  pixmapwid = gtk_pixmap_new(pixmap, mask);			   
  gtk_box_pack_start(GTK_BOX(box), pixmapwid, FALSE,FALSE,0);
  gtk_widget_show(pixmapwid);	
  
  label = gtk_label_new("Nessusd Host : ");
  gtk_box_pack_end(GTK_BOX(box), label,FALSE,FALSE,0);
  gtk_widget_show(label);
  
  box = gtk_hbox_new(FALSE,0);
  gtk_table_attach_defaults(GTK_TABLE(table), box, 0,1,4,5);
  gtk_widget_show(box);
  
  pixmap = gdk_pixmap_create_from_xpm_d(entries.window->window, &mask,
					&style->bg[GTK_STATE_NORMAL],
					user_xpm);             
  pixmapwid = gtk_pixmap_new(pixmap, mask);	   
  gtk_box_pack_start(GTK_BOX(box), pixmapwid, FALSE,FALSE,0);
  
  label = gtk_label_new("Login :");
  gtk_box_pack_end(GTK_BOX(box), label, FALSE, FALSE,0);
  gtk_widget_show(label);
  gtk_widget_show(pixmapwid);
 	  
  gtk_widget_show(button);
#ifdef USE_AF_INET
  gtk_widget_show(entries.hostname);
  gtk_widget_show(entries.port);
#endif
  gtk_widget_show(entries.username);
  gtk_widget_show(entries.password);
  gtk_widget_show(table);
  gtk_widget_show(entries.window);
  gtk_main();
  return(0);
}
 
void
auth_dialog_cancel(w, widget)
   GtkWidget * w, *widget;
{
 nessusd_running = 0;
 close_window(w,widget);
}

int
auth_dialog_close(w,ww)
 GtkWidget * w, *ww;
{
 nessusd_running = 0;
 return(FALSE);
}

/* 
 * auth_dialog_callback
 *
 * This function is called whenever the user has
 * clicked on the 'OK' button of the authentification
 * dialog.
 */
void 
auth_dialog_callback(widget, entries)
     GtkWidget * widget;
     struct auth_entries * entries;
{
  char * username;
  char * password;
#ifdef USE_AF_INET
  char * hostname;
  int port;
#endif
  char * t;
  char * err;
  
#ifdef USE_AF_INET
  t = gtk_entry_get_text(GTK_ENTRY(entries->hostname));
  if(!strlen(t))
    {
      show_error("You must enter an hostname");
      return;
    }
 hostname = emalloc(strlen(t)+1);
 strncpy(hostname, t, strlen(t));
 t = gtk_entry_get_text(GTK_ENTRY(entries->port));
 if(!strlen(t))
   {
     show_error("You must enter a valid port number !");
     return;
   }
 port = atoi(t);
 if((port < 0) || (port > 65536))
   {
     show_error("Your port specification is illegal");
     return;
   }
#endif
  t = gtk_entry_get_text(GTK_ENTRY(entries->username));
  if(!strlen(t))
    {
      show_error("You must enter a valid username");
      return;
    }
  username = emalloc(strlen(t)+1);
  strncpy(username, t, strlen(t));
 
  t = gtk_entry_get_text(GTK_ENTRY(entries->password));
  if(!strlen(t))
    {
      show_error("You must enter a valid password");
      return;
    }
  password = emalloc(strlen(t)+1);
  strncpy(password, t, strlen(t));
#ifdef USE_AF_INET
  err = connect_to_nessusd(hostname, port, username, password);
#else
  err = connect_to_nessusd("localhost", -1, username, password);
#endif
  if(err)show_error(err);
  gtk_widget_hide(entries->window);
  if(!err)
#ifdef USE_AF_INET
    attack_dialog_setup(hostname);
#else 
  attack_dialog_setup("localhost");
#endif
else nessusd_running = 0;
}












