/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */

#include <config.h>
#include <nessuslib.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include <gtk/gtk.h>
#include <unistd.h>
#include <sys/un.h>
#include <netinet/in.h> 
#include <signal.h>

#include "auth_dialog.h"
#include "xstuff.h"
#include "comm.h"
#include "auth.h"
#include "attack.h"
#include "report.h"
#include "parser.h"
#include "report_file.h"
#include "error_dialog.h"
#include "main_window.h"
#include "sighand.h"
#include "preferences.h"


struct arglist * plugins;
struct arglist * hosts;
struct arglist * prefs;
int nessusd_running;
int quiet_mode;




char * connect_to_nessusd(char *, int, char *, char *);
void init_globals();

/*
 * connect_to_nessusd
 *
 * This function establishes the connection between
 * nessus and nessusd, logs in and reads the plugin
 * list from the server.
 *
 */
char *
connect_to_nessusd(hostname, port, login, password)
	char * hostname;
	int port;
	char * login;
	char * password;
	
{
  int soc;
#ifndef USE_AF_INET
  struct sockaddr_un address;
  char * name = AF_UNIX_PATH;
#endif

  init_globals();
#ifdef USE_AF_INET
  soc = open_sock_tcp_hn(hostname, port);
  if(soc<0)
  	{
  	struct in_addr a = nn_resolve(hostname);
  	if(!a.s_addr)return("Host not found !");
  	else
  	return("Could not open a connection to the remote host");
  	}
#else
  if((soc = socket(AF_UNIX, SOCK_STREAM,0))==-1){
  	perror("socket ");
  	exit(1);
  	}
  bzero(&address, sizeof(struct sockaddr_un));
  address.sun_family = AF_UNIX;
  bcopy(name, address.sun_path, strlen(name));
  if(connect(soc, &address, sizeof(address))==-1){
  	return("Could not open a connection to the remote host");
  	}
#endif  
  if(comm_init(soc) || (auth_login(login, password)))
  	return("Remote host is not using NTP/1.0");
  	
  if(comm_get_pluginlist())return("Login failed");
  return(NULL);
}

/*
 * init_globals
 *
 * initializes two main global variables : plugins and
 * hosts
 */
void 
init_globals()
{
  plugins = emalloc(sizeof(struct arglist));
  hosts = emalloc(sizeof(struct arglist));
}


void 
display_help()
{
 printf("nessus, version %s\n", NESSUS_VERSION);
#ifdef USE_AF_INET
 printf("\nusage : nessus [-vh] [-q host port user target result]\n\n");
#else
 printf("\nusage : nessus [-vh] [-q user target result]\n\n");
#endif
 printf("\tv : shows version number\n");
 printf("\th : shows this help\n"); 
 printf("\tq : quiet mode : nessus performs the test without displaying anything\n");
 printf("\t    to the screen.\n\n");
 printf("\tThe quiet mode arguments are :\n");
#ifdef USE_AF_INET
 printf("\t\thost     : nessusd host\n");
 printf("\t\tport     : nessusd host port\n");
#endif
 printf("\t\tuser     : user name\n");
 printf("\t\ttarget   : target host\n");
 printf("\t\tresult   : name of the file where \n\t\t\t   nessus will store the results\n");
}
 
 
void main(int argc, char * argv[])
{
  int i;
  init_display(&argc, argv);
  signal(SIGSEGV , sighand_sigsegv);
  preferences_init(&prefs);
  quiet_mode = 0;
  while((i=getopt(argc, argv, "vhq"))!=EOF)
  {
   switch(i)
   {
   
    case 'v' :
    	printf("nessus (%s) %s -- (c) 1998 Renaud Deraison <deraison@worldnet.fr>\n", 
    			PROGNAME,NESSUS_VERSION);
    	exit(0);
    	break;
    	
    case 'q' : 
        {
#ifdef USE_AF_INET      
        char * nessusd_host;
        char * port;
#endif
        char * username;
        char * password;
        char * target;
        char * fname;
        char * err;
        int j;
        
    	quiet_mode = 1;
#ifdef USE_AF_INET
    	for(j=0;j<5;j++)
#else
	for(j=0;j<3;j++)
#endif
    	{
    	 if(!argv[optind+j]){
    	 	display_help();
    	 	exit(0);
    	 	}
    	}
#ifdef USE_AF_INET
    	nessusd_host = argv[optind++];
    	port = argv[optind++];
#endif
    	username = argv[optind++];
    	target = argv[optind++];
    	fname = argv[optind++];
    	password = getpass("Password : ");
#ifdef USE_AF_INET
	if((err = connect_to_nessusd(nessusd_host, atoi(port), username, password)))
#else
	if((err = connect_to_nessusd("localhost", 1, username, password)))
#endif	
	{
	 fprintf(stderr, "nessus : %s\n", err);
	 exit(0);
	}
	if(fork()==0)
	{
	attack_host(target, -1, -1);
	{
	 int finished = 0;
	 int type;
	 char * buf;
	 char * msg;
	 buf = emalloc(4096);
	 msg = emalloc(4096);
	 
	 while(!finished)
	 {
	  auth_gets(buf, 4095);
	  buf[strlen(buf)-1]=0;
	  if((type = parse_server_message(buf,hosts, msg))==MSG_BYE)
	  	finished = 1;
	  bzero(msg, 4095);
	 }
	}
	arglist_to_file(hosts, fname);
	}
	exit(0);
	}
    default :
    	display_help();
    	exit(0);
    	break;
    }
  } 
  nessusd_running = 0;
  main_window_setup();
  gtk_main();
  shutdown(0,2);
  close_display();
  exit(0);
}

 
