/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 */
 
 
#include <config.h>
#include <nessuslib.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include <gtk/gtk.h>

#include "xstuff.h"
#include "plugin_infos.h"
#include "plugin_list.h"
#include "families.h"
#include "preferences.h"
struct plugin_list_data {
  	GtkWidget * families_list;
  	GtkWidget * plugins_list;
	};
	
extern struct arglist * prefs;
struct plugin_families * families;
extern struct arglist * plugins;
static int plugin_listed = 0;

void plugin_list_close(GtkWidget *  , GtkWidget *);
void family_toggle_callback(GtkWidget * , struct plugin_list_data * );
void plugin_list_toggle_callback(GtkWidget * , struct arglist * );
void plugin_list_callback(GtkWidget * , GtkWidget * );
void family_list_callback(GtkWidget * , struct plugin_list_data * );

/*
 * plugin_list_setup
 *
 * Draws the main window showing informations
 * about the plugins of the server
 */
void 
plugin_list_setup()
{
 GtkWidget * window;
 GtkWidget * families_window;
 GtkWidget * plugins_window;
 GtkWidget * w_box;
 GtkTooltips * tooltips;
 GtkWidget * c_button;
 struct plugin_list_data * data;
 GList * dlist = NULL;

 struct arglist * plugs = plugins;
 struct plugin_families * lfamilies;
 
 
 if(plugin_listed)return;
 data = emalloc(sizeof(struct plugin_list_data));
 
 lfamilies = families = family_init();
 
 while(plugs && plugs->next)
 {
  family_add(families, plugs->value);
  plugs = plugs->next;
 }
 plugs = plugins;
 
 plugin_listed = 1;
 window = gtk_window_new(GTK_WINDOW_TOPLEVEL); 
 gtk_signal_connect(GTK_OBJECT(window),"delete_event",
  GTK_SIGNAL_FUNC(delete_event),window);
  
 gtk_window_set_title(GTK_WINDOW(window), "Installed plugins");
 gtk_container_border_width(GTK_CONTAINER(window), 10);
 
 w_box = gtk_vbox_new(FALSE, 15);
 gtk_container_add(GTK_CONTAINER(window), w_box);
 
 families_window = gtk_scrolled_window_new(NULL,NULL);
 gtk_container_border_width(GTK_CONTAINER(families_window), 10);
 gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(families_window),
 	GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 gtk_widget_set_usize(families_window, 400, 200);
 gtk_box_pack_start(GTK_BOX(w_box), families_window, TRUE, FALSE, 0);
 
 plugins_window = gtk_scrolled_window_new(NULL,NULL);
 gtk_container_border_width(GTK_CONTAINER(plugins_window), 10);
 gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(plugins_window),
 	GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 gtk_widget_set_usize(plugins_window, 400, 200);
 gtk_box_pack_start(GTK_BOX(w_box), plugins_window, TRUE, FALSE, 0);
 
 
 c_button = gtk_button_new_with_label("Close");
 gtk_box_pack_end(GTK_BOX(w_box), c_button, FALSE, FALSE, 0);
 gtk_signal_connect(GTK_OBJECT(c_button),"clicked",
  (GtkSignalFunc)plugin_list_close,window);
 gtk_widget_show(c_button);
 
 data->families_list = gtk_list_new();
 
 
 gtk_container_add(GTK_CONTAINER(families_window), data->families_list);
 gtk_widget_show(data->families_list);
 gtk_signal_connect(GTK_OBJECT(data->families_list),
                              "selection_changed",
                              GTK_SIGNAL_FUNC(family_list_callback),
                              data);
                              
  data->plugins_list = gtk_list_new();                          
  gtk_container_add(GTK_CONTAINER(plugins_window), data->plugins_list);
  gtk_widget_show(data->plugins_list);
  gtk_signal_connect(GTK_OBJECT(data->plugins_list),
  				"selection_changed",
  				GTK_SIGNAL_FUNC(plugin_list_callback),     
  				data->plugins_list);                      
                             
 
 tooltips = gtk_tooltips_new();

 while(lfamilies && lfamilies->next)
 {
	GtkWidget * item;
	GtkWidget * box;
	GtkWidget * button;
	GtkWidget * label;
	
 	item = gtk_list_item_new();
	
	button = gtk_toggle_button_new_with_label(" ");
	gtk_widget_set_usize(button, 15, 15);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), 
		family_enabled(lfamilies->name, plugins) ? TRUE:FALSE);
	label = gtk_label_new(lfamilies->name);
	box = gtk_hbox_new(FALSE,5);
	gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
	gtk_widget_show(label);
	gtk_box_pack_end(GTK_BOX(box), button, FALSE, FALSE,0);
	
	gtk_container_add(GTK_CONTAINER(item), box);
	gtk_signal_connect(GTK_OBJECT(button),
                              "clicked",
                              GTK_SIGNAL_FUNC(family_toggle_callback),
                              data);
     
	gtk_widget_show(button);
	gtk_widget_show(box);
	gtk_widget_show(item);
 	dlist = g_list_append(dlist, item);
 	gtk_object_set_data(GTK_OBJECT(item),
                                   "list_item_data",
                                   lfamilies->name);
 	lfamilies = lfamilies->next;
 }
 
 gtk_tooltips_enable(tooltips);
 gtk_list_append_items(GTK_LIST(data->families_list), dlist);
 gtk_widget_show(families_window);
 gtk_widget_show(plugins_window);
 gtk_widget_show(w_box);
 gtk_widget_show(window);
}
 
void 
plugin_list_close(widget,w)
  GtkWidget * widget; 
  GtkWidget *w;
{
	if(arg_get_value(prefs, "remember_plugin_set"))
	 {
	  preferences_save(plugins);
	  pluginset_reload(plugins);
	 }
	gtk_widget_hide(w);
	plugin_listed = 0;
}


void plugin_list_info_close(w,ww)
   GtkWidget * w, *ww;
{
	gtk_widget_hide(ww);
}


void 
plugin_list_callback(widget, list)
  GtkWidget * widget;
  GtkWidget * list;
{
 GtkObject * list_item;
 char * cur_plug;
 GList * dlist;
 
 dlist = GTK_LIST(list)->selection;
 if(!dlist)return;
 
 list_item = GTK_OBJECT(dlist->data);
 cur_plug = gtk_object_get_data(list_item,"list_item_data");
 plugin_info_window_setup(cur_plug);             
}

void family_toggle_callback(w, data)
 GtkWidget * w;
 struct plugin_list_data * data;
{
 int enable = GTK_TOGGLE_BUTTON (w)->active;
 GList * dlist;
 GtkObject * list_item;
 GtkObject * button;
 struct arglist * plugin;
 char * name;
 GtkWidget * item;
 
 item = w->parent->parent;
 
 list_item = GTK_OBJECT(item);
 name = gtk_object_get_data(list_item,"list_item_data");
 family_enable(name, plugins, enable); 
 dlist = GTK_LIST(data->plugins_list)->children;
 if(dlist)
 {
 list_item = GTK_OBJECT(dlist->data);
 
 plugin = gtk_object_get_data(list_item,"plugin");
 if(plug_get_family(plugin) && 
 	!strcmp(plug_get_family(plugin), name))
 {
 while(dlist)
 {
  list_item = GTK_OBJECT(dlist->data);
  button = gtk_object_get_data(list_item,"button");
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), enable);
  dlist = dlist->next;
 }
 }
 }
}

void plugin_list_toggle_callback(w, plugin)
 GtkWidget * w;
 struct arglist * plugin;
{
 int state = (int)plug_get_launch(plugin);
 plug_set_launch(plugin,state ? 0:1);
}

void family_list_callback(widget, data)
  GtkWidget * widget; 
  struct plugin_list_data * data;
{
 GList * dlist;
 GtkObject * list_item;
 char * cur_family;
 struct arglist * plugs = plugins;
 GtkTooltips * tooltips;
 
 dlist = GTK_LIST(data->families_list)->selection;
 if(!dlist)return;
 
 list_item = GTK_OBJECT(dlist->data);
 cur_family = gtk_object_get_data(list_item,"list_item_data");
 dlist = GTK_LIST(data->plugins_list)->children;
 if(dlist)gtk_list_remove_items(GTK_LIST(data->plugins_list),dlist); 
 dlist = NULL;
 tooltips = gtk_tooltips_new();
 while(plugs && plugs->next)
 {
	GtkWidget * item;
	GtkWidget * box;
	GtkWidget * button;
	GtkWidget * label;
	
  	 
	if(plug_get_family(plugs->value) &&
		!strcmp(plug_get_family(plugs->value), cur_family))
	{
	
 	item = gtk_list_item_new();
	button = gtk_toggle_button_new_with_label(" ");
	gtk_widget_set_usize(button, 15, 15);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), 
		(int)plug_get_launch(plugs->value));
	
	label = gtk_label_new(plugs->name);
	box = gtk_hbox_new(FALSE,5);
	gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
	gtk_widget_show(label);
	
	gtk_box_pack_end(GTK_BOX(box), button, FALSE, FALSE,0);
	gtk_container_add(GTK_CONTAINER(item), box);
	gtk_signal_connect(GTK_OBJECT(button),
                              "clicked",
                              GTK_SIGNAL_FUNC(plugin_list_toggle_callback),
                              plugs->value);
	gtk_widget_show(button);
	gtk_widget_show(box);
	gtk_widget_show(item);
	if(arg_get_value(plugs->value, "SUMMARY"))
	gtk_tooltips_set_tip(tooltips, item, 
		(gchar *)arg_get_value(plugs->value, "SUMMARY"),"");
 	dlist = g_list_append(dlist, item);
 	gtk_object_set_data(GTK_OBJECT(item),
                                   "list_item_data",
                                   plugs->name);
         gtk_object_set_data(GTK_OBJECT(item),
                                   "button",
                                    button);     
         gtk_object_set_data(GTK_OBJECT(item),
         			   "plugin",
         			   plugs->value);                                        
         }                    
 	plugs = plugs->next;	
 }
  gtk_tooltips_enable(tooltips);
  gtk_list_append_items(GTK_LIST(data->plugins_list), dlist);
}





  
