/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Preferences  -- maps the content of the nessusd.conf file to memory
 *
 */
 
#include <config.h>
#include <nessuslib.h>
#include <nessus-devel.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <errno.h>
#include <pwd.h>

#include "preferences.h"

extern struct arglist * prefs;

int prefs_buffer_parse(char *, struct arglist *);
int prefs_add_subcategory(struct arglist *, FILE *);
char * preferences_get_filename();

int preferences_init(prefs)
	struct arglist ** prefs;
{
  char * filename;
  int result;
  *prefs = emalloc(sizeof(struct arglist));
  filename = preferences_get_filename();
  result = preferences_process(filename, *prefs);
  efree(filename);
  return(result);
}

char * 
preferences_get_filename()
{
  char* home;
  char * ret;
  struct passwd * pwd;
  
  home = getenv("HOME");
  if(home)
  {
   ret = emalloc(strlen(home)+strlen("/.nessusrc")+1);
   sprintf(ret, "%s/.nessusrc", home);
   return(ret);
  }
  pwd = getpwuid(getuid());
  if (pwd && pwd->pw_dir)
  {
   ret = emalloc(strlen(pwd->pw_dir)+strlen("/.nessusrc")+1);
   sprintf(ret, "%s/.nessusrc", pwd->pw_dir);
   return(ret);
  } 
  return(NULL);
}
 
int preferences_new()
{
  int fd;
  char * buf;
  char * fn = preferences_get_filename();
  if(!fn)return(-1);
  
   if((fd = open(fn, O_CREAT | O_RDWR, 0600))<0){
    perror("open ");
    return(-1);
  }
  buf = emalloc(500);
  sprintf(buf,"# Nessus Client Preferences File\n\
scan_hosts = yes\n\
scan_range = 1-8001\n\
remember_plugin_set  = yes\n");
  write(fd, buf, strlen(buf));
  efree(buf);
  close(fd);
  efree(fn);
  return(0);
}

int preferences_process(filename,prefs)
     char * filename;
     struct arglist * prefs;
{
  FILE * fd;
  char * buffer;
    if(filename)
      {
	if(!(fd = fopen(filename, "r"))) {
	 if(errno == EACCES)
	 {
	  fprintf(stderr,
	  	"The Nesssus client doesn't have the right to read %s\n", filename);
	  exit(1);
	 }
#ifdef DEBUG
	  printf("Couldn't find any prefs file... Creating a new one...\n");
#endif 
	  if(preferences_new()){
	    fprintf(stderr, "Error creating %s : ", preferences_get_filename());
	    perror("open ");
	    return(1);
	  }
	  else
	    if(!(fd = fopen(filename, "r")))
	      {
	        perror("open ");
		printf("Could not open %s -- now quitting\n", filename);
		exit(2);
	      }
	}
	buffer = emalloc(255);
	while(!feof(fd) && fgets(buffer, 254,fd))
	  {
	   if(strchr(buffer, '='))
	    prefs_buffer_parse(buffer, prefs);
	   else if(!strncmp(buffer, "begin(", strlen("begin(")))
	   {
	    char * t = buffer+(strlen("begin(")*sizeof(char));
	    char * end = strchr(t, ')');
	    char * category_name;
	    
	    if(!end)
	    {
	     fprintf(stderr, "Parse error in %s : %s\n", filename, buffer);
	    }
	    else
	    {
	     struct arglist * subcategory;
	     end[0]=0;
	     category_name = emalloc(strlen(t)+1);
	     strncpy(category_name, t, strlen(t));
	     subcategory = emalloc(sizeof(struct arglist));
	     if(prefs_add_subcategory(subcategory, fd))
	       fprintf(stderr, "Missing 'end' in %s\n", filename);
	     else
	     arg_add_value(prefs, category_name, ARG_ARGLIST, -1, subcategory);
	     }
	    }
     	 }
	 fclose(fd);
    return(0);
    }
   else return(1);
}
 
 
int 
prefs_add_subcategory(arglist, fd)
 struct arglist * arglist;
 FILE * fd;
{
 char * buffer = emalloc(255);
 int flag = 0;
 
 while(!flag && !feof(fd) && fgets(buffer, 254,fd))
 {
  if(!strlen(buffer))return(1);
  if(!strncmp(buffer, "end", 3))flag = 1;
  else
    prefs_buffer_parse(buffer, arglist);
  bzero(buffer, 255);
 }
 efree(buffer);
 return(0);
}
   
int 
prefs_buffer_parse(buffer, arglist)
 char * buffer;
 struct arglist * arglist;
{
 char * t;
 char * opt;
 char * value;
 int val = -1;
 if(buffer[strlen(buffer)-1]=='\n')buffer[strlen(buffer)-1]=0;
 if(buffer[0]=='#')return(1);
 opt = buffer;
 if((t = strchr(buffer, '=')))
 {
  t[0]=0;
  t+=sizeof(char);
  while(t[0]==' ' && t[0])t+=sizeof(char);
  if(!t[0])return(1);
  while(opt[strlen(opt)-1]==' ')opt[strlen(opt)-1]=0;
  if(!strcmp(t, "yes"))val = 1;
  if(!strcmp(t, "no"))val = 0;
  if(val == -1)
  {
   value=emalloc(strlen(t)+1);
   strncpy(value, t, strlen(t));
   arg_add_value(arglist, opt, ARG_STRING, strlen(value), value);
  }
  else {
  	arg_add_value(arglist, opt, ARG_INT, sizeof(int), (void *)val);
	}
  return(0);
 }
 else return(1);
}

void
preferences_save(plugins)
 struct arglist * plugins;
{
 FILE * fd;
 char * filename = preferences_get_filename();
 struct arglist * t;
 fd = fopen(filename , "w");
 if(!fd)
  {
  fprintf(stderr, "%s could not be opened write only\n", filename);
  efree(filename);
  return;
  }
 fprintf(fd, "# This file was automagically created by nessus\n");
 t = prefs;
 while(t && t->next)
 {
  if((int)t->type==ARG_INT)
   fprintf(fd, "%s = %s\n", t->name, t->value?"yes":"no");
  else if(t->type == ARG_STRING)fprintf(fd, "%s = %s\n", t->name, (char *)t->value);
  t = t->next;
 }
 if(arg_get_value(prefs, "remember_plugin_set") && plugins && plugins->next)
 {
  t = plugins;
  fprintf(fd, "\n\nbegin(plugin_set)\n");
  while(t && t->next)
   { 
    fprintf(fd, "%s = %s\n", t->name, (plug_get_launch(t->value))?"yes":"no");
    t = t->next;
   }
  fprintf(fd, "end\n");
 }
  fclose(fd);
}


void
pluginset_apply(plugins)
 struct arglist * plugins;
{
 while(plugins && plugins->next)
 {
  int flag = 0;
  struct arglist * t = arg_get_value(prefs, "plugin_set");
  if(!t)return;
  while(!flag && t && t->next)
  {
   if(!strcmp(plugins->name, t->name))
    {
     plug_set_launch(plugins->value, (int)t->value);
     flag = 1;
    }
    t = t->next;
   }
   plugins = plugins->next;
  }
 }
 
void
pluginset_reload(plugins)
 struct arglist * plugins;
{
 while(plugins && plugins->next)
 {
  int flag = 0;
  struct arglist * t = arg_get_value(prefs, "plugin_set");
  if(!t)return;
  while(!flag && t && t->next)
  {
   if(!strcmp(plugins->name, t->name))
   {
    t->value = plug_get_launch(plugins->value);
    flag = 1;
   }
  t = t->next;
  }
  plugins = plugins->next;
 }
}
