/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <config.h>
#include <nessuslib.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef HAVE_STRING_H
#include <string.h>
#endif /* HAVE_STRING_H */
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include <gtk/gtk.h>
#include "xstuff.h"
#include "preferences.h"

struct prefs_entries
{
 GtkWidget * window;
 GtkWidget * port_scan_button;
 GtkWidget * port_scan_range;
 GtkWidget * remember_plugin_set_button;
};


extern struct arglist * prefs;
extern struct arglist * plugins;

void prefs_dialog_callback(GtkWidget * , struct prefs_entries *);
 
/*
 * prefs_dialog_setup
 * 
 * This function draws the preferences dialog of the Nessus
 * client
 *
 */
void 
prefs_dialog_setup()
{
 GtkWidget * label;
 GtkWidget * button;
 GtkWidget * table;
 GtkWidget * item;
 struct prefs_entries entries;
 
 entries.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 gtk_window_set_title(GTK_WINDOW(entries.window), "Nessus Preferences");
 gtk_signal_connect(GTK_OBJECT(entries.window), "delete_event", 
 	GTK_SIGNAL_FUNC(delete_event), 0);

 gtk_container_border_width(GTK_CONTAINER(entries.window), 10);
 
 table = gtk_table_new(6,2,TRUE);
 gtk_table_set_col_spacings(GTK_TABLE(table),5); 
 gtk_table_set_row_spacings(GTK_TABLE(table),5); 
 gtk_container_add(GTK_CONTAINER(entries.window), table);
 
 label = gtk_label_new("Nessus Preferences : ");
 gtk_table_attach_defaults(GTK_TABLE(table), label, 0,2,0,1);
 gtk_widget_show(label);
 
 entries.port_scan_button = gtk_check_button_new_with_label("do a portscan first");
 gtk_table_attach_defaults(GTK_TABLE(table), entries.port_scan_button, 
 				0, 2, 1,2);
 gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(entries.port_scan_button), 
 		(int)arg_get_value(prefs, "scan_hosts"));
 gtk_widget_show(entries.port_scan_button);
 
 label = gtk_label_new("Portscan range : ");
 gtk_table_attach_defaults(GTK_TABLE(table), label, 0,1,2,3);
 gtk_widget_show(label);
 
 entries.port_scan_range = gtk_entry_new();
 gtk_table_attach_defaults(GTK_TABLE(table), entries.port_scan_range, 1,2, 2,3);
 gtk_entry_set_text(GTK_ENTRY(entries.port_scan_range), 
 	arg_get_value(prefs,"scan_range")?
		arg_get_value(prefs,"scan_range"):"1-8001");
 gtk_widget_show(entries.port_scan_range);
 
 entries.remember_plugin_set_button = 
 	gtk_check_button_new_with_label("Remember plugins set");
 gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(entries.remember_plugin_set_button), 
 		(int)arg_get_value(prefs, "remember_plugin_set"));
 gtk_table_attach_defaults(GTK_TABLE(table), entries.remember_plugin_set_button,
 				0,2,3,4);
 gtk_widget_show(entries.remember_plugin_set_button);
 
 item = gtk_hseparator_new();
 gtk_table_attach_defaults(GTK_TABLE(table), item, 0,2,4,5);
 gtk_widget_show(item);
 
 button = gtk_button_new_with_label("Cancel");
 gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(close_window),
 	entries.window);
 gtk_table_attach_defaults(GTK_TABLE(table), button, 0,1,5,6);
 gtk_widget_show(button);
 
 button = gtk_button_new_with_label("Save");
 gtk_signal_connect(GTK_OBJECT(button), "clicked", (GtkSignalFunc)prefs_dialog_callback,
 	(void *)&entries);
 gtk_table_attach_defaults(GTK_TABLE(table), button, 1,2,5,6);
 gtk_widget_show(button);

 gtk_widget_show(table);
 gtk_widget_show(entries.window);
 gtk_main();
}


void
prefs_dialog_callback(w,entries)
 GtkWidget * w;
 struct prefs_entries * entries;
{
 int do_scan;
 int do_save_sets;
 char * scan_range, * copy;
 struct arglist * t;
 
 do_scan = GTK_TOGGLE_BUTTON(entries->port_scan_button)->active;
 if(arg_set_value(prefs, "scan_hosts", sizeof(int), (void *)do_scan)==-1)
 {
  arg_add_value(prefs, "scan_hosts", ARG_INT, sizeof(int), (void *)do_scan);
 }
 
 do_save_sets = GTK_TOGGLE_BUTTON(entries->remember_plugin_set_button)->active;
 if(arg_set_value(prefs, "remember_plugin_set", sizeof(int), (void *)do_save_sets)==-1)
 {
  arg_add_value(prefs, "remember_plugin_set", ARG_INT, sizeof(int), (void *)do_save_sets);
 }
 
 scan_range = gtk_entry_get_text(GTK_ENTRY(entries->port_scan_range));
 copy = emalloc(strlen(scan_range)+1);
 strncpy(copy, scan_range, strlen(scan_range));
 if(arg_set_value(prefs, "scan_range", strlen(copy), copy)==-1)
  {
  arg_add_value(prefs, "scan_range", ARG_STRING, strlen(copy), copy);
  }
 preferences_save(plugins);
 gtk_widget_hide(w->parent->parent);
 
}
