/* Nessus
 * Copyright (C) 1998 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Attack.c : 
 *  launch the plugins, and manages the multithreading 
 *
 */
 
 
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h> 
#include <time.h>
#include <sys/time.h>
#include <unistd.h>
#include <signal.h>
#ifdef HAVE_UCBINCLUDE
#include "/usr/ucbinclude/fcntl.h"
#else
#include <fcntl.h>
#endif /* HAVE_UCBINCLUDE */
#include <sys/ioctl.h>
#include <errno.h>

#include <config.h>
#include <nessus-devel.h>
#include <nessuslib.h>
#include "attack.h"
#include "log.h"
#include "hostloop.h"
#include "sighand.h"
#include "rules.h"
#include "auth.h"
#include "threads.h"
#include "comm.h" 

#ifndef INADDR_NONE
#define INADDR_NONE 0xffffffff
#endif

extern int nmap_main(struct arglist *, char *);
extern struct arglist * plugins;
extern struct arglist * preferences;
extern struct nessusd_threads *nessusd_threads; 
int attack_num_daemons;
pid_t current_plugin;

/*
 * Setup an arglist from a given hostname
 */
struct arglist * 
attack_init_hostinfos(hostname)
     char * hostname;
{
  struct arglist * hostinfos;
  struct arglist * ports;
  struct in_addr *p_addr;
  
  p_addr = emalloc(sizeof(struct in_addr));
  *p_addr = nn_resolve(hostname);
  if((p_addr->s_addr == INADDR_NONE) || (p_addr->s_addr == INADDR_ANY))
    {
      efree(p_addr);
      return(NULL);
    }
  hostinfos = emalloc(sizeof(struct arglist));
  arg_add_value(hostinfos, "FQDN", ARG_STRING, strlen(hostname), hostname);
  arg_add_value(hostinfos, "IP", ARG_PTR, sizeof(struct in_addr), p_addr);
  ports = emalloc(sizeof(struct arglist));
  arg_add_value(hostinfos, "PORTS", ARG_ARGLIST, sizeof(struct arglist), ports);
  return(hostinfos);
}


/*
 * Add the informations about a host in a plugin arglist
 */
int 
attack_add_hostinfos(plugin, hostinfos)
     struct arglist *  plugin;
     struct arglist *  hostinfos;
{
  struct arglist * args;
  struct arglist * t;
  
  args = arg_get_value(plugin->value, "plugin_args");
  if(!args)
    {
#ifdef DEBUG
      log_write("error : a plugin has no \"plugin_args\" value !\n");
      log_write("dumping arglist\n");
      arg_dump(plugin->value, 0);
#endif
      return(-1);
    }
  
  t = arg_get_value(plugin, "HOSTNAME");
  if(t)arg_set_value(args, "HOSTNAME", sizeof(struct arglist), hostinfos);
  else arg_add_value(args, "HOSTNAME", ARG_ARGLIST, sizeof(struct arglist), hostinfos);
  return(0);
}

/*
 * This function attacks *one* host
 */
void 
attack_host_thread(hostinfos, plugins, hostname, port_range)
     struct arglist * hostinfos;
     struct arglist * plugins;
     char * hostname;
     char * port_range;
{ 
  int num_plugs = 0;
  int cur_plug = 1;
  struct arglist * t;
 
  t = plugins;
  while(t && t->next)
  { 
  if(plug_get_launch(t->value))num_plugs++;
   t = t->next;
  }
  nmap_main(hostinfos, port_range);
  while(plugins && plugins->next)
    {
      plugin_run_t f_run;
      struct arglist * args;
      if(plug_get_launch(plugins->value))
       {
        if(attack_add_hostinfos(plugins, hostinfos))
	  continue;
      f_run = (plugin_run_t)arg_get_value(plugins->value, "plugin_run");
      args = arg_get_value(plugins->value, "plugin_args");
#ifdef DEBUG
      printf("Launching %s\n", plugins->name);
#endif
#ifdef LOGMORE
      log_write("launching %s against %s\n", plugins->name,
      	hostname);
#endif
      comm_send_status(hostname, cur_plug, num_plugs);
      cur_plug++;
      current_plugin = fork();
      if(current_plugin==0)
       {
       if(args)(*f_run)(args);
       exit(0);
       }
     else 
     	{
     	signal(SIGALRM, sighand_alarm_plugin);
     	alarm(PLUGIN_TIMEOUT);
     	waitpid(current_plugin, NULL, 0);
     	signal(SIGALRM, SIG_IGN);
     	alarm(0);
     	}
      }
      plugins = plugins->next;
    }
}


/*
 * This is the main function for attacking
 * a network
 */
int 
attack_host(hostname, port_range,max_hosts)
     char * hostname;
     char * port_range;
     int max_hosts;
{
  struct arglist * hostinfos;
  int max_threads;
  struct arglist * lplugs = plugins;
  struct arglist * plugs;
  struct arglist * tplugs;
  int i;
  int num_tested = 0;
  struct nessusd_threads * t;
  int flag;
  
  if(!(geteuid()==0))
  {
   auth_printf("SERVER <|> ERROR <|> The server is not suid root, some attacks won't work <|> SERVER\n");
  }
   
  tplugs = plugs = emalloc(sizeof(struct arglist));
  
  /*
   * Make our socket non-blocking 
   */
  signal(SIGIO, sighand_io);
  fcntl(0, F_SETOWN, getpid());
  fcntl(0, F_SETFL, FASYNC);
  
  /*
   * Put the plugins in the right order 
   */
  for(i=ACT_GATHER_INFO;i<=ACT_DENIAL;i++)
  {
  while(lplugs && lplugs->next)
  {
   struct arglist * args;
   
   args = arg_get_value(lplugs->value, "plugin_args");
   if(args && ((int)arg_get_value(args, "CATEGORY")==i))
   {
   tplugs->value = lplugs->value;
   tplugs->name = lplugs->name;
   tplugs->type = lplugs->type;
   tplugs->length = lplugs->length;
   tplugs->next = emalloc(sizeof(struct arglist));
   tplugs = tplugs->next;
   }
   lplugs = lplugs->next;
  }
  lplugs = plugins;
 } 
 
 log_write("new attack against %s\n", hostname);
 
 if(arg_get_value(preferences, "max_threads"))
    {
      max_threads = atoi(arg_get_value(preferences, "max_threads"));
      if(max_threads<=0)
	{
	  log_write("Error ! max_threads = %d -- check "NESSUSD_CONF"\n", 
		    max_threads);
	  max_threads = 1;
	}
    }
  else max_threads = 5;
  
  attack_num_daemons = 0;
  /*
   * Initialize the hostloop system
   */
  hls_new_session(hostname);
  while(hostname && num_tested != max_hosts)
    {
      pid_t pid;
      
      flag = 0 ;
      if(pTEST(get_hostname_attributes(hostname)))
      {
       if(attack_num_daemons < max_threads)
	{
	  if((pid=fork())==0) 
	    {
	      hostinfos = attack_init_hostinfos(hostname);
	      if(!hostinfos)
	        auth_printf("SERVER <|> ERROR <|> %s : host unknown <|> SERVER\n", 
	       		hostname);
	      else
	        attack_host_thread(hostinfos, plugs, hostname, port_range); 
	      exit(0); 
	    }
	  else { 
	  	nessusd_thread_register(hostname, pid);
	  	attack_num_daemons++;
	  	log_write("testing %s\n", hostname);
		} 
	}
      else 
      	{
      	flag = 1;
	while(attack_num_daemons >= max_threads)
	 {
	 struct nessusd_threads * t = nessusd_threads;
	 while(t && t->next)
	  {
	   if(t->up)
	   {
	   int status = 0;
	   int code;
	   code = waitpid(t->pid, &status, WNOHANG);
	   if((status && WIFEXITED(status)) || code==ECHILD || code==-1)
	   	{
		nessusd_thread_unregister(t->pid);
		attack_num_daemons--;
		}
	   }
	   t = t->next;
	 }
        }
       }
    }
      if(!flag){
        num_tested ++;
       	hostname = hls_get_next_host(); 
       	}
   }
  
   flag = 1;
   while(flag)
   {
    flag = 0;
    t = nessusd_threads;
    while(t && t->next)
    {
     if(t->up)
     {
      int status = 0;
      int code;
	   
      flag = 1;
      code = waitpid(t->pid, &status, WNOHANG);
      if((status && WIFEXITED(status)) || code==-1)
       {
	nessusd_thread_unregister(t->pid);
	attack_num_daemons--;
       }
      }
      t = t->next;
    }
   }
   
  /*
   * Clean up some stuff in memory 
   */
  tplugs = plugs;
  while(tplugs)
  {
   tplugs = plugs->next;
   efree(plugs);
   plugs = tplugs;
  }
  return(0);
}
   
   
   
 
